from .pornemby_answer import _PornembyAnswerAnswerMonitor

__ignore__ = True


class TestPornembyExamMonitor(_PornembyAnswerAnswerMonitor):
    name = "Pornemby 科举 测试"
    chat_name = "api_group"
    chat_user = "embykeeper_test_bot"
    chat_allow_outgoing = True
